<template>

  <div
      :class="`fixed top-0 left-0 w-full flex items-center h-full z-50 transition duration-200 ease ${visible ? 'visible' : 'invisible'}`">
    <div
        :class="`absolute flex items-center justify-center left-0 top-0 bg-black bg-opacity-25 w-full h-full transition-all duration-200 ${visible ? 'opacity-100' :'opacity-0'}`"/>
    <button v-if="visible" @click="visible = false" class="btn btn-circle btn-primary absolute right-4 top-4 p-3">
      <svg xmlns="http://www.w3.org/2000/svg" class="h-6 w-6" fill="none" viewBox="0 0 24 24"
           stroke="currentColor">
        <path stroke-linecap="round" stroke-linejoin="round" stroke-width="2" d="M6 18L18 6M6 6l12 12"/>
      </svg>
    </button>

    <div
        v-if="object.sticker"
        class="relative max-w-2/3 p-3 w-auto h-auto m-auto flex items-center justify-center overflow-hidden">
      <div
          style="max-height: 90vh"
          :class="`relative block w-full bg-white overflow-hidden rounded-lg transform transition duration-200 ${visible ? '-translate-y-0 opacity-100 visible' : 'translate-y-full opacity-0 invisible'}`">
        <button
            @click="downloadImg(helpers.generateImage(object?.sticker?.sticker_url))"
            :class="`btn btn-circle btn-primary absolute right-2 top-2 p-3 ${downloadLoading ? 'loading' : ''}`">
          <svg v-show="!downloadLoading" class="relative block w-full h-full" viewBox="0 0 18 18" fill="none"
               xmlns="http://www.w3.org/2000/svg">
            <path
                d="M16 10V15C16 15.55 15.55 16 15 16L3 16C2.45 16 2 15.55 2 15L2 10C2 9.45 1.55 9 1 9C0.45 9 0 9.45 0 10L0 16C0 17.1 0.9 18 2 18L16 18C17.1 18 18 17.1 18 16L18 10C18 9.45 17.55 9 17 9C16.45 9 16 9.45 16 10ZM10 9.67L11.88 7.79C11.9726 7.69742 12.0825 7.62398 12.2035 7.57387C12.3244 7.52377 12.4541 7.49798 12.585 7.49798C12.7159 7.49798 12.8456 7.52377 12.9665 7.57387C13.0875 7.62398 13.1974 7.69742 13.29 7.79C13.3826 7.88258 13.456 7.99249 13.5061 8.11346C13.5562 8.23442 13.582 8.36407 13.582 8.495C13.582 8.62593 13.5562 8.75558 13.5061 8.87654C13.456 8.99751 13.3826 9.10742 13.29 9.2L9.7 12.79C9.60749 12.8827 9.4976 12.9563 9.37662 13.0064C9.25565 13.0566 9.12597 13.0824 8.995 13.0824C8.86403 13.0824 8.73435 13.0566 8.61338 13.0064C8.4924 12.9563 8.38251 12.8827 8.29 12.79L4.7 9.2C4.51302 9.01302 4.40798 8.75943 4.40798 8.495C4.40798 8.23057 4.51302 7.97698 4.7 7.79C4.88698 7.60302 5.14057 7.49798 5.405 7.49798C5.66943 7.49798 5.92302 7.60302 6.11 7.79L8 9.67L8 1C8 0.45 8.45 0 9 0C9.55 0 10 0.45 10 1L10 9.67Z"
                fill="white"/>
          </svg>
        </button>
        <img draggable="false" class="h-screen w-auto block object-center" crossorigin="anonymous"
             :src="helpers.generateImage(object?.sticker?.sticker_url)" alt=""/>
      </div>
    </div>
  </div>
    <!-- <div class="chat-image avatar sticky top-0 bottom-0">
      <div :class="`w-10 rounded-full ${isLoader ? 'bg-gray-200 animate-pulse' : ''}`">
        <ic-person-general v-show="!isLoader"/>
      </div>
    </div> -->

  <MenuReaction v-if="isShowReaction && showEmoji && status !== 'failed'" :id="id" :is-seller="isSeller" @onSuccess="handleReactionSuccess" @onActive="handleOnActive"/>
  <MenuChatBubble v-if="isShowMenuChat" :is-seller="isSeller" :message="message" :id="id" :chat_type="chatType" :object="object"/>
  <ChatRepliedMessage v-if="reply_from_id && object?.replied_message  && status !== 'loading'" :replied-message="object?.replied_message" class="cursor-pointer" @click="viewReply()"/>
  <ReactionIcon :reaction-list="reactionArr"/>


  <div
        class="relative cursor-pointer group mb-4 rounded-md overflow-hidden block h-auto aspect-auto w-100">
    <div v-if="isLoading"
          class="absolute left-1/2 top-1/2 transform -translate-x-1/2 -translate-y-1/2 m-auto z-10 flex items-center justify-center bg-opacity-20 transition duration-200 hover:bg-opacity-30 select-none bg-black w-full h-full">
      <svg aria-hidden="true" class="w-8 h-8 mr-2 text-gray-200 animate-spin fill-blue-600"
            viewBox="0 0 100 101" fill="none" xmlns="http://www.w3.org/2000/svg">
        <path
            d="M100 50.5908C100 78.2051 77.6142 100.591 50 100.591C22.3858 100.591 0 78.2051 0 50.5908C0 22.9766 22.3858 0.59082 50 0.59082C77.6142 0.59082 100 22.9766 100 50.5908ZM9.08144 50.5908C9.08144 73.1895 27.4013 91.5094 50 91.5094C72.5987 91.5094 90.9186 73.1895 90.9186 50.5908C90.9186 27.9921 72.5987 9.67226 50 9.67226C27.4013 9.67226 9.08144 27.9921 9.08144 50.5908Z"
            fill="currentColor"/>
        <path
            d="M93.9676 39.0409C96.393 38.4038 97.8624 35.9116 97.0079 33.5539C95.2932 28.8227 92.871 24.3692 89.8167 20.348C85.8452 15.1192 80.8826 10.7238 75.2124 7.41289C69.5422 4.10194 63.2754 1.94025 56.7698 1.05124C51.7666 0.367541 46.6976 0.446843 41.7345 1.27873C39.2613 1.69328 37.813 4.19778 38.4501 6.62326C39.0873 9.04874 41.5694 10.4717 44.0505 10.1071C47.8511 9.54855 51.7191 9.52689 55.5402 10.0491C60.8642 10.7766 65.9928 12.5457 70.6331 15.2552C75.2735 17.9648 79.3347 21.5619 82.5849 25.841C84.9175 28.9121 86.7997 32.2913 88.1811 35.8758C89.083 38.2158 91.5421 39.6781 93.9676 39.0409Z"
            fill="currentFill"/>
      </svg>
    </div>
    <div
        class="absolute top-0 left-0 w-full h-full bg-black group-hover:opacity-10 opacity-0 transition duration-200 z-20"></div>
    <img
        v-if="object.sticker"
        class="m-auto w-auto h-auto object-cover"
        crossorigin="anonymous" :src="helpers.generateImage(object.sticker.sticker_url)"/>
  </div>
  <span v-html="message" class="overflow-hidden relative block break-words"></span>
  <FailedSendMessage v-if="isSeller && status ==='failed'" :message="wa_error_message"/>
  <div :class="`relative flex items-end gap-5 ${assignee ? 'justify-between' : 'justify-end'}` ">
    <div class="flex gap-1 items-center justify-end">
      <span class="block  text-xs text-gray-600 text-right select-none">{{ date }}</span>
      <!-- loading -->

      <template v-if="isSeller && !isLoader">
        <IcMessageStatus :status="status"/>
      </template>
      <!--      <span class="block absolute right-2 bottom-1 text-xs text-gray-600 mt-2 text-right select-none">{{ date }}</span>-->
    </div>
  </div>
  <vue-easy-lightbox
      :visible="visibleRef"
      :imgs="imgsRef"
      :index="indexRef"
      @hide="onHide"></vue-easy-lightbox>
</template>

<script setup lang="ts">
import {onMounted, ref, watch} from "vue";
import {useChatListStore} from "../../stores/chat_list";
import {useGeneralStore} from "../../stores/general";
import helpers from "../../utils/helpers";
import IcPersonGeneral from "../Icon/IcPersonGeneral.vue";
import FailedSendMessage from "../Utility/FailedSendMessage.vue";
import IcMessageStatus from "../Icon/IcMessageStatus.vue";
import axios from "axios";
import ChatRepliedMessage from "./ChatRepliedMessage.vue";
import MenuChatBubble from "../Menu/MenuChatBubble.vue";
import ReactionIcon from "../Icon/ReactionIcon.vue";
import MenuReaction from "../../components/Menu/MenuReaction.vue";

const visibleRef = ref(false)
const downloadLoading = ref(false)
const indexRef = ref(0) // default 0
const imgsRef = ref(null)
const visible = ref<boolean>(false);
const setVisible = (value): void => {
  visible.value = value;
};

interface Props {
  isSeller?: boolean
  isLoader?: boolean
  isError?: boolean
  isShowReaction?: boolean
  isShowMenuChat?: boolean
  isLoading?: boolean
  date?: string
  id?: bigint
  status?: string
  wa_error_message?: string
  message?: string
  chatType?: string
  assignee?: object | null
  object?: any
  reply_from_id?: any
  reactions?: any
}

let props = withDefaults(defineProps<Props>(), {
  isSeller: false,
  isLoader: false,
  isShowReaction: true,
  isShowMenuChat: true,
  isError: false,
  isLoading: false,
  date: null,
  id: null,
  wa_error_message: null,
  status: null,
  assignee: null,
  chatType: null,
  message: null,
  reply_from_id: null,
  reactions: [],
  object: {
    replied_message: {
      message: null,
      name: null,
    },
    sticker: {
      sticker_url: null
    }
  },
});
const reactionArr = ref([])
const chat = useChatListStore()
const general = useGeneralStore()
const showEmoji = ref(false)
const emojiActive = ref(false)

const handleReactionSuccess = (emojiString: any) => {
  let type_chat = general.chatTab
  let platform = ''

  if(general.chatTab === 'instagram'){
    type_chat = 'instagram'

    if(chat.selectedChatType === 'instagram'){
      platform = 'instagram'
    }
    
    if(chat.selectedChatType === 'facebook'){
      platform = 'page'
    }
  }

  chat.sendEmoji(chat.selectedChat, {
    to: chat?.detail?.customer?.phone,
    chat_id: props.id,
    emoji: emojiString,
    platform: platform
  },type_chat).then((resp) => {
    if (resp.code === 200) {
      const currentReaction = reactionArr.value.findIndex((rc) => rc.chat_id === props.id && rc.from_chat_user_id === chat.selectedChat)
      if (currentReaction == -1 && reactionArr.value.length == 0) {
        reactionArr.value.push({
          chat_id : props?.id,
          from_chat_user_id: chat.selectedChat,
          value: emojiString
        })
      } else {

        reactionArr.value.splice(currentReaction,1, {
          chat_id : props?.id,
          from_chat_user_id: chat.selectedChat,
          value: emojiString
        })
      }
    } else {
      console.log("400")
    }
  })
}

const viewReply= () => {
  general.isShowReplyChat = !general.isShowReplyChat

  if(general.isShowReplyChat == true){
    general.isShowProfile = false
    general.replyChatId = props?.id
  }else{
    general.replyChatId = null
  }
}

const handleOnActive= (event) => {
  emojiActive.value = event
}

onMounted(() => {
  reactionArr.value = props.reactions
})

watch(() => props.reactions, (val) => {
  reactionArr.value = val
})
const downloadImg = async (url) => {
  downloadLoading.value = true
  const link = document.createElement('a');
  link.href = url;
  link.setAttribute('download', url.substring(url.lastIndexOf('/') + 1)); //or any other extension
  document.body.appendChild(link);
  link.click();
  downloadLoading.value = false
}

const showPreviewImage = () => {
  chat.previewImage = true
  chat.previewImageData = {
    id: props.id,
    object: props.object
  }
}
const onHide = () => {
  visibleRef.value = false
}
const onShow = () => {
  visibleRef.value = true
}
const showSingle = () => {
  imgsRef.value = props.object.sticker.sticker_url
  // or
  // imgsRef.value  = {
  //   title: 'this is a placeholder',
  //   src: 'http://via.placeholder.com/350x150'
  // }
  onShow()
}
onMounted(() => {
})
// watch(() => props.object.sticker.sticker_url, (val) => {
// console.log(val, "isi image on change")
// })
</script>

<style lang="css">
.ant-image-preview-img {
  display: inline-block !important;
}

</style>
